@@ -1,9 +1,7 @@
 /*
- * jsimd_arm64.c
- *
  * Copyright 2009 Pierre Ossman <ossman@cendio.se> for Cendio AB
  * Copyright (C) 2011, Nokia Corporation and/or its subsidiary(-ies).
- * Copyright (C) 2009-2011, 2013-2014, 2016, 2018, 2020, 2022, 2024,
+ * Copyright (C) 2009-2011, 2013-2014, 2016, 2018, 2020, 2022, 2024-2025,
  *           D. R. Commander.
  * Copyright (C) 2015-2016, 2018, 2022, Matthieu Darbois.
  * Copyright (C) 2020, Arm Limited.
@@ -975,7 +973,7 @@ jsimd_can_huff_encode_one_block(void)
   if (sizeof(JCOEF) != 2)
     return 0;
 
-  if (simd_support & JSIMD_NEON && simd_huffman)
+  if ((simd_support & JSIMD_NEON) && simd_huffman)
     return 1;
 
   return 0;
@@ -1010,7 +1008,7 @@ jsimd_can_encode_mcu_AC_first_prepare(void)
   if (SIZEOF_SIZE_T != 8)
     return 0;
 
-  if (simd_support & JSIMD_NEON)
+  if ((simd_support & JSIMD_NEON) && simd_huffman)
     return 1;
 
   return 0;
@@ -1037,7 +1035,7 @@ jsimd_can_encode_mcu_AC_refine_prepare(void)
   if (SIZEOF_SIZE_T != 8)
     return 0;
 
-  if (simd_support & JSIMD_NEON)
+  if ((simd_support & JSIMD_NEON) && simd_huffman)
     return 1;
 
   return 0;
